<?php

require 'vendor/autoload.php';
require_once("$CFG->libdir/formslib.php");

require_once($CFG->dirroot . '/course/modlib.php');

use CL\Helper;

class block_campuslabs_course_sync extends block_base
{

    function init()
    {
        $this->title = get_string('title', 'block_campuslabs_course_sync');
    }

    function get_required_javascript() {
        parent::get_required_javascript();

        $this->page->requires->jquery();
    }

    function get_content()
    {
        // If content is cached.
        if ($this->content !== null) {
            return $this->content;
        }

        global $CFG;

        $context = context_course::instance($this->page->course->id);

        if (!has_capability('block/campuslabs_course_sync:view', $context)) {
            return;
        }

        $this->content = new stdClass;
        $config = get_config('campuslabs_course_sync');
        if ($config->oauth_client_id === "" || $config->oauth_client_secret === "") {
            $this->content->text= '<p>You do not have course sync properly installed.  Please contact your administrator.</p>';
        } else {
            $this->content->text = '
            <script>
            $(document).ready(function() {
                $(".campuslabs_rubrics_form").submit(function(event) {
                    event.preventDefault();
                    var form = $(this);
                    sendToCampusLabs(form)
                });

                $("#campuslabs_launch_rubrics").submit(function(event) {
                    event.preventDefault();
                    var form = $(".campuslabs_rubrics_form"),
                        rubricsWindow = window.open("'.$CFG->wwwroot . '/blocks/'.Helper::BUILDING_BLOCK_NAME.'/syncing_rubrics.html", "_blank");

                    sendToCampusLabs(form, function(hasError) {
                        setTimeout(function() {
                        		
                        	if(hasError)
                        		rubricsWindow.location.replace("'.$CFG->wwwroot . '/blocks/'.Helper::BUILDING_BLOCK_NAME.'/syncing_error.html");
                        	else
                            	rubricsWindow.location.href = "'.$CFG->wwwroot . '/mod/lti/view.php?id='.$this->get_lti_id().'";
                            			
					    }, 2000);

                    });
                })
            });

            function sendToCampusLabs(form, cb) {
                var timing = 300;
                var hasError = false;
                $(".campuslabs_syncing").fadeIn(timing);
                $(".campuslabs_sync-button button").prop("disabled",true);

                $.get(form.attr("action"), form.serialize())
                   .done(function() {
                        $(".campuslabs_syncing").delay(500).fadeOut(timing, function() {
                            // Show Success Message
                            $("#campuslabs_syncSuccess").fadeIn(timing);
                            $(".campuslabs_sync-button button").delay(5000).prop("disabled",false);
                        });
                   })
                   .fail(function(err) {
                            			
                    hasError = true;
                    $(".campuslabs_syncing").delay(500).fadeOut(timing, function() {
                        switch (err.status) {
                            case 404:
                                if ($.parseJSON( err.responseText).message === "cl_course_not_found") {
                                    $("#campuslabs_noCourseFound").fadeIn(timing);
                                } else {
                                    $("#campuslabs_permissionDenied").fadeIn(timing);
                                }
                                break;
                           case 401:
                                $("#campuslabs_permissionDenied").fadeIn(timing);
                                break;
                           default:
                               $("#campuslabs_syncError").fadeIn(timing);
                               break;
                       }
                    });
                   })
                   .always(function() {
                    $(".campuslabs_sync-button button").delay(5000).prop("disabled",false);
                    setTimeout(function() {
                            $("#campuslabs_login").fadeOut(timing);
                            $("#campuslabs_permissionDenied").fadeOut(timing);
                            $("#campuslabs_noCourseFound").fadeOut(timing);
                            $("#campuslabs_syncSuccess").fadeOut(timing);
                            $("#campuslabs_syncError").fadeOut(timing);
                    }, 5000);
                    if (cb) {
                        cb(hasError);
                    }
                });
            }
           </script>

        <div class="campuslabs_course_sync">

            <div class="campuslabs_alert campuslabs_error" id="campuslabs_syncError">
                <span>An error has occurred.  Please contact <a href="mail:support@campuslabs.com">support@campuslabs.com</a></span>
            </div>

            <div class="campuslabs_alert error"  id="campuslabs_noCourseFound">
                <span>You must synchronize your course data before you can retrieve rubric scores.</span>
            </div>

            <div class="campuslabs_alert campuslabs_error" id="campuslabs_permissionDenied">
                <span>You do not have permission to sync this course.</span>
            </div>

            <div class="campuslabs_alert campuslabs_success" id="campuslabs_syncSuccess">
                <span>Course data synchronization is complete.</span>
            </div>

            <p>Course Sync will send your current enrollment and assignment information to
            <a target="_blank" href="https://rubricssupport.campuslabs.com/hc/en-us/articles/360044401512">Campus Labs Rubrics</a>
            so you can evaluate course participants and sync your grades back into Moodle.
            </p>

            <div class="campuslabs_sync-button">
                <form class="campuslabs_rubrics_form" id="campuslabs_sync_datacl" action="' . $CFG->wwwroot . '/blocks/'.Helper::BUILDING_BLOCK_NAME.'/course_section_sync.php" style="display:inline">
                    <fieldset class="invisiblefieldset">
                        <input name="course_id" type="hidden" value="' . $this->page->course->id . '" />
                        <input type="hidden" name="sesskey" value="'.sesskey().'" />
                        <button id="searchform_button" type="submit" title="Sync Data">Sync Data</button>
                    </fieldset>
                </form>
            </div>

            <div class="campuslabs_sync-button">
                <form class="campuslabs_rubrics_form" action="' . $CFG->wwwroot . '/blocks/'.Helper::BUILDING_BLOCK_NAME.'/grade_sync.php" style="display:inline">
                    <fieldset class="invisiblefieldset">
                        <input name="course_id" type="hidden" value="' . $this->page->course->id . '" />
                        <input type="hidden" name="sesskey" value="'.sesskey().'" />
                        <button id="searchform_button" type="submit" title="Get Scores">Get Grades</button>
                    </fieldset>
                </form>
            </div>

            <div class="campuslabs_syncing">
                <img src="' . $CFG->wwwroot . '/blocks/'.Helper::BUILDING_BLOCK_NAME.'/assets/loading.gif" class="campuslabs_spinner" />
            </div>
            <div class="campuslabs_logo">
                <img src="' . $CFG->wwwroot . '/blocks/'.Helper::BUILDING_BLOCK_NAME.'/assets/cl-logo-tagline.png" />
            </div>

        </div>';

        }

        return $this->content;
    }

    public function has_config()
    {
        return true;
    }

    public function applicable_formats()
    {
        return array('course-view' => true);
    }

    //gets called when the building block is installed on the page
    function instance_create()
    {
        global $DB;

        //Determine if there is a lti module on the course page
        $ltiLaunchId = $DB->get_field_sql("SELECT distinct ml.id FROM {lti} ml where ml.name = 'Rubrics' and ml.course = ?", [$this->page->course->id]);

        //Determine if the admin setup the lti link
        $adminLtiId = $DB->get_field_sql("SELECT id FROM {lti_types} where name = 'Campus Labs Rubrics'");

        if (!empty($adminLtiId) && empty($ltiLaunchId)) {
            $ltiModule = $this->add_lti_module_to_course_page($adminLtiId);
            $context = context_module::instance($ltiModule->coursemodule);

            //Remove student permissions from viewing the module
            role_change_permission(5, $context, "mod/lti:view", CAP_PREVENT);

            //Removes teacher and non editing teacher from removing the block manually
            role_change_permission(4, $context, "moodle/course:manageactivities", CAP_PREVENT);
            role_change_permission(3, $context, "moodle/course:manageactivities", CAP_PREVENT);
        }
    }

    //remove the building block module from the course page when the building block is removed
    function instance_delete()
    {
        $courseModuleId = $this->get_lti_id();

        if (!empty($courseModuleId)) {
            course_delete_module($courseModuleId);
        }
    }

    private function get_lti_id()
    {
        global $DB;
        return $DB->get_field_sql("SELECT id FROM {course_modules} where idnumber = 'cl_lti_rubrics_module' and course=?", [$this->page->course->id]);
    }

    private function add_lti_module_to_course_page($adminLtiId)
    {
        global $DB;

        $course = $DB->get_record('course', array('id' => $this->page->course->id), '*', MUST_EXIST);
        $module = $DB->get_record('modules', array('name' => 'lti'), '*', MUST_EXIST);

        $data = new stdClass();
        $data->section = 0;
        $data->visible = 1;
        $data->course = $course->id;
        $data->module = $module->id;
        $data->modulename = $module->name;
        $data->cmidnumber = 'cl_lti_rubrics_module';
        $data->add = 'lti';
        $data->showtitlelaunch = 1;
        $data->launchcontainer = 5;
        $data->typeid = $adminLtiId;
        $data->name = "Rubrics";
        $data->completion = 0;
        $data->completionview = 0;
        $data->completionexpected = 0;

        return add_moduleinfo($data, $course);
    }
}
