<?php
//Moodle's files
require(__DIR__ . '/../../config.php');
require(__DIR__ . '/../../mod/assign/locallib.php');

//CL and dependencies
require 'vendor/autoload.php';

global $PAGE;

use CL\Daos\ScoreSyncDao;
use CL\Models\CourseNotFoundException;

$courseId = required_param('course_id', PARAM_INT);

require_login($courseId);
require_sesskey();
require_capability('block/campuslabs_course_sync:view', context_course::instance($courseId));

$PAGE->set_context(context_course::instance($courseId));

$scoreSyncDao = new ScoreSyncDao;

try {
    $scores = $scoreSyncDao->syncFromCampusLabs($courseId);
} catch (CourseNotFoundException $e) {
    http_response_code(404);
    echo json_encode(['message' => 'cl_course_not_found']);
    return false;
} catch (\Exception $e) {
  $syncError = true;
}

$response = $scoreSyncDao->sendSyncedGradesToCampusLabs($courseId, $scores);

//Javascript is disabled
if (empty($_SERVER['HTTP_X_REQUESTED_WITH']) || strtolower($_SERVER['HTTP_X_REQUESTED_WITH']) != 'xmlhttprequest') {
    $redirectUrl = $CFG->wwwroot . '/course/view.php?id=' . $courseId;
    $redirectTimeInSeconds = 5;

    if (!$response->error && !$syncError) {
        redirect($redirectUrl, "Successfully synced grades to Campus Labs. Redirecting to course page.", $redirectTimeInSeconds);
    } else {
        redirect($redirectUrl, "An error has occurred.  Please contact support@campuslabs.com", $redirectTimeInSeconds);
    }
} else {
    http_response_code($response->http_status_code ?: 500);
}

