<?php

namespace CL\Daos;

use CL\Helper;
use CL\Models\CourseNotFoundException;
use CL\Models\CourseSection;
use CL\Models\Score;
use Curl\Curl;

class CampusLabsDao
{
    private $caCertPath;

    public function __construct()
    {
        global $CFG;
        $this->caCertPath = $CFG->dirroot."/blocks/".Helper::BUILDING_BLOCK_NAME."/cacert.pem";
    }

    /**
     * @param CourseSection $courseSection
     * @return Curl
     */
    public function sendCourseSectionToCampusLabs(CourseSection $courseSection)
    {
        $request = new Curl();
        $request->setHeader("Content-Type", "application/json");
        $request->setHeader("Authorization", "Bearer ".$this->getAuthToken());
        $request->setOpt(CURLOPT_CAINFO, $this->caCertPath);
        $request->post(Helper::COURSESECTION_URL, $courseSection->toJson());
        return $request;
    }

    /**
     * @param int $courseId
     * @param Score[] $scores
     * @return Curl
     */
    public function sendSyncedGradesToCampusLabs($courseId, $scores)
    {
        $request = new Curl();
        $request->setHeader("Content-Type", "application/json");
        $request->setHeader("Authorization", "Bearer ".$this->getAuthToken());
        $request->setOpt(CURLOPT_CAINFO, $this->caCertPath);
        $request->post(sprintf(Helper::GRADE_URL, $courseId), Score::toJson($scores));
        return $request;
    }

    /**
     * @param int $courseId
     * @return Curl
     * @throws CourseNotFoundException
     * @throws \Exception
     */
    public function getScoresByCourse($courseId)
    {
        $request = new Curl();
        $request->setHeader("Authorization", "Bearer ".$this->getAuthToken());
        $request->setOpt(CURLOPT_CAINFO, $this->caCertPath);
        $request->get(sprintf(Helper::GRADE_URL, $courseId));

        if ($request->http_status_code === 404) {
            throw new CourseNotFoundException();
        }

        if ($request->error) {
            throw new \Exception;
        }

        return $request;
    }

    /**
     * @return string
     */
    private function getAuthToken()
    {
        $request = new Curl();
        $request->setOpt(CURLOPT_CAINFO, $this->caCertPath);
        return $request->post(Helper::TOKEN_AUTH_URL,
            $data = http_build_query(array(
                'client_id' => \get_config('campuslabs_course_sync', 'oauth_client_id'),
                'client_secret' => \get_config('campuslabs_course_sync', 'oauth_client_secret'),
                'grant_type' => Helper::TOKEN_GRANT_TYPE
            ), '', '&')
        )->access_token;
    }
}