<?php

namespace CL\Daos;

use CL\Models\Score;

class ScoreSyncDao
{
    /**
     * @var CampusLabsDao
     */
    private $clDao;

    public function __construct()
    {
        $this->clDao = new CampusLabsDao;
    }

    /**
     * @param $courseId int id of course
     * @return Score[]
     */
    public function syncFromCampusLabs($courseId)
    {
        $response = $this->clDao->getScoresByCourse($courseId);
        $scores = Score::fromJsonToScore($response->response);

        foreach ($scores as $score) {
            $cm = \get_coursemodule_from_instance('assign', $score->AssignmentIdentifier, 0, false, MUST_EXIST);
            $context = \context_module::instance($cm->id);

            (new \assign($context, $cm, null))->save_grade($score->EnrollmentIdentifier, $this->buildGrade($score));
            $score->LastSynced = (new \DateTime)->format('c');
        }

        return $scores;
    }

    /**
     * @param int $courseId
     * @param Score[] $scores
     * @return Curl
     */
    public function sendSyncedGradesToCampusLabs($courseId, $scores)
    {
        return $this->clDao->sendSyncedGradesToCampusLabs($courseId, $scores);
    }

    /**
     * @param Score $score
     * @return \stdClass
     */
    private function buildGrade(Score $score)
    {
        $gradeData = new \stdClass();
        $gradeData->addattempt = false;
        $gradeData->applytoall = false;
        $gradeData->assignfeedbackcomments_editor = ['format' => 1, 'text' => $score->Comments];
        $gradeData->attemptnumber = -1;
        $gradeData->grade = (float)$score->Grade;
        $gradeData->workflowstate = ASSIGN_MARKING_WORKFLOW_STATE_READYFORRELEASE;
        return $gradeData;
    }
}